#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Mindmap to Bitbucket Markdown."""
from freeplane_tools.base import BaseNode, MindMapInterface
import re


class BitBucketNode(BaseNode):
    """Bitbucket Node class."""

    @property
    def _toc_name(self):
        plc = re.sub("[^a-zA-Z0-9 ]", "", self.text).lower().split()
        plc = "-".join(plc)
        return plc

    def format(self):
        if self.list_depth >= 0:
            return self.list_format()
        lines = []
        title_pad = "#" * (self.depth + 1)
        title = self.text
        img_link = ""
        if self.link and self.has_img_link:
            img_link = self.img_link()
        elif self.link:
            title = f"{self.text} [\[link\]]({self.link})"
        title = f"{title_pad} {title} [&#8593;](#toc)"
        lines.append(f'<a name="{self._toc_name}"></a>')
        lines.append(title)
        lines.append(self.note)
        if img_link:
            lines.append(img_link)
        return "\n".join(lines)


class MindMap2StashMarkup(MindMapInterface):
    node = BitBucketNode

    def get_document(self):
        txt = "\n".join(
            [
                '<a name="toc"></a>',
                "# TOC",
                self._toc(),
                "",
                "",
                self._body(),
                "",
            ]
        )
        return txt



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Base class."""
from bs4 import Tag, BeautifulSoup
from freeplane_tools.mm import Mindmap
import re

FLAG_LIST = "child:list"
IMG_EXTS = [
    ".jpg",
    ".png",
    ".svg",
]


class BaseNode(object):
    """Base Node Class."""

    def __init__(self, tag: Tag):
        self.tag = tag
        self.text = tag.attrs.get("text")
        self.link = tag.attrs.get("link")
        self.depth = len(tag.findParents("node"))

    @property
    def note(self):
        note = self.tag.find("richcontent", recursive=False)
        if not note:
            return ""
        lines = []
        for i in note.findAll("p"):
            txt = i.getText().strip("\n").rstrip()
            txt = txt.replace("```", "\n```\n")
            txt = re.sub("^      ", "", txt)
            lines.append(txt)
        retval = "\n".join(lines)
        return retval

    @property
    def list_depth(self):
        for parent in self.tag.findParents("node"):
            t = self.__class__(parent)
            if FLAG_LIST in t.flags:
                return self.depth - t.depth - 1
        return -1

    @property
    def flags(self):
        val = self.tag.findChildren("attribute", recursive=False)
        for i in val:
            yield ":".join(i.attrs.values())

    def list_format(self):
        pad = " " * 3 * self.list_depth
        out = " ".join(i for i in self.text.split("\n"))
        if self.link and self.has_img_link:
            out = " ".join(
                [
                    self.text,
                    self.img_link(self.text),
                ]
            )
        elif self.link:
            out = f"[{self.text}]({self.link})"
        return f"{pad}* {out}"

    def img_link(self, desc=None):
        if not desc:
            return f"![{self.link}]({self.link})"
        return f'![{self.link}]({self.link} "{desc}")'

    @property
    def has_img_link(self):
        if self.link:
            return any(self.link.lower().endswith(i) for i in IMG_EXTS)
        return False

    @property
    def _toc_name(self):
        raise RuntimeError(f"Child class is missing _toc_name method")

    @property
    def toc_entry(self):
        if self.list_depth != -1:
            return ""
        pad = " " * 3 * (self.depth) + "* "
        return f"{pad}[{self.text}](#{self._toc_name})"

    def format(self):
        raise RuntimeError(f"Child class is missing format method")


class MindMapInterface(object):
    # Needs to be defined in subclass
    node = None

    def __init__(self, srcfile):
        self.mm = Mindmap(srcfile)
        if self.__class__.node is None:
            raise RuntimeError(f"Please define {__class__}.node")

    def _iternodes(self):
        for obj in self.mm.iternodes():
            yield self.__class__.node(obj)

    def _toc(self):
        return "\n".join(n.toc_entry for n in self._iternodes() if n.toc_entry)

    def _body(self):
        return "\n".join(n.format() for n in self._iternodes())

    def get_document(self):
        txt = "\n".join(
            [
                '<a name="toc"></a>',
                "# TOC",
                self._toc(),
                "",
                "",
                self._body(),
                "",
            ]
        )
        return txt

    def write_document(self, dstfile):
        with open(dstfile, "w") as fileh:
            fileh.write(self.get_document())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Copy a Mindmap Template to destination file."""
from freeplane_tools.data import get_template
import argparse
import re

parser = argparse.ArgumentParser(description=__doc__)
parser.add_argument(
    "dest_file",
    help="destination file",
    type=str,
)


def main(args):
    """Run main function."""
    if not re.match(r".*\.mm$", args.dest_file):
        raise SystemExit(f"Destination file needs to end with .mm")
    content = get_template()
    with open(args.dest_file, "wb") as fileh:
        print(f"Writing to file: {args.dest_file}")
        fileh.write(content)


if __name__ == "__main__":
    args = parser.parse_args()
    # args = parser.parse_args(["./test.mm"])
    main(args)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Mindmap Template."""
import pathlib

DATADIR = pathlib.Path(__file__).resolve().parent.joinpath("data")


def get_template():
    return DATADIR.joinpath("template.mm").read_bytes()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Convert a Freeplane Mindmap to Github Markdown."""
from freeplane_tools.github import MindMap2GithubMarkdown
import argparse
import re

parser = argparse.ArgumentParser(description=__doc__)
parser.add_argument(
    "mindmap_file",
    help="mindmap_file help",
    type=str,
)
parser.add_argument(
    "-w",
    "--write",
    action="store_true",
    help="write markdown file",
    default=False,
)
parser.add_argument(
    "-o",
    "--outfile",
    type=str,
    required=False,
    default=None,
)


def main(args):
    """Run main function."""
    mm = MindMap2GithubMarkdown(args.mindmap_file)
    if args.write:
        outfile = args.outfile or re.sub(r".mm$", ".md", args.mindmap_file)
        if args.mindmap_file == outfile:
            raise SystemExit(f"Error, {outfile} is the same as {args.mindmap_file}")
        print(f"Saving file to: {outfile}")
        mm.write_document(outfile)
    else:
        print(mm.get_document(), flush=True)


if __name__ == "__main__":
    args = parser.parse_args()
    if not re.match(r".*\.mm$", args.mindmap_file):
        raise SystemExit("Invalid file, needs to be a mindmap file")
    # args = parser.parse_args(["./README.mm", "-w"])
    main(args)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from setuptools import setup
from glob import glob


setup(
    name="freeplane_tools",
    install_requires=[
        "beautifulsoup4",
        "lxml",
    ],
    scripts=list(glob("./bin/*.py")) + list(glob("./bin/*.sh")),
)
